package org.examples;

import org.antlr.v4.runtime.Token;
import org.examples.CalculatorParser.*;

import java.util.Map;
import java.util.Stack;

public class ArithmeticListener extends CalculatorBaseListener
{
    private Stack<Double>       stack  = new Stack<Double>();
    private Map<String, Double> sym ;
    private Double ret;

    public ArithmeticListener(Map<String, Double> sym) {
        this.sym = sym;
    }

    public Double getResult()
    {
        return this.ret;
    }

    @Override
    public void exitIdentifier(IdentifierContext ctx)
    {
        if (!sym.containsKey(ctx.ID().getText())) {
            Token node = ctx.ID().getSymbol();
            throw new UndefineVariableException(
                    "line " + node.getLine() + ":" + node.getCharPositionInLine() +
                    " variable '" + node.getText() + "' is undefined"
            );
        }
        stack.push(sym.get(ctx.ID().getText()));
    }

    @Override
    public void exitFloat(FloatContext ctx)
    {
        stack.push(Double.valueOf(ctx.FLOAT().getText()));
    }

    @Override
    public void exitAddOrSubtract(AddOrSubtractContext ctx)
    {
        Double op1 = stack.pop();
        Double op2 = stack.pop();

        String op = ctx.getChild(1).getText();
        if ("-".equals(op)) {
            stack.push(op2 - op1);
        } else if ("+".equals(op)) {
            stack.push(op2 + op1);
        }
    }

    @Override
    public void exitLine(LineContext ctx) {
        ret = stack.pop();
    }

    @Override
    public void exitNegExpr(NegExprContext ctx) {
        stack.push(-stack.pop());
    }

    @Override
    public void exitMultOrDiv(MultOrDivContext ctx)
    {
        Double op1 = stack.pop();
        Double op2 = stack.pop();

        String op = ctx.getChild(1).getText();
        if ("/".equals(op)) {
            stack.push(op2 / op1);
        } else if ("*".equals(op)) {
            stack.push(op2 * op1);
        }
    }
}