package org.examples;

import org.antlr.v4.runtime.ANTLRInputStream;
import org.antlr.v4.runtime.CommonTokenStream;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import java.util.HashMap;
import java.util.Map;

public class CalculatorTest {

    private Map<String, Double> sym = new HashMap<String, Double>();
    @Before
    public void setVariable() {
        sym.put("PI", Math.PI);
        sym.put("radius", 5.0);
    }

    @Test
    public void test1() throws Exception {
        Double ans = calc("2 * PI * radius", sym);
        Assert.assertEquals(ans, 2 * Math.PI * sym.get("radius"), 0.0001);
    }

    @Test(expected = UndefineVariableException.class)
    public void test2() throws Exception {
        calc("2 * -C",sym);
    }

    @Test
    public void test3() throws Exception {
        Double ans = calc("radius * - radius", sym);
        Assert.assertEquals(ans, - sym.get("radius") * sym.get("radius"), 0.0001);
    }

    private Double calc(String expression, Map<String, Double> sym) throws Exception{
        /* 根据表达式创建lexer */
        CalculatorLexer lexer = new CalculatorLexer(new ANTLRInputStream(expression));

        /* 根据lexer 创建token stream */
        CommonTokenStream tokens = new CommonTokenStream(lexer);

        /* 根据token stream 创建 parser */
        CalculatorParser paser = new CalculatorParser(tokens);

        /* 为parser添加一个监听器 */
        ArithmeticListener listener = new ArithmeticListener(sym);
        paser.addParseListener(listener);

        /* 匹配 line 监听器会记录结果 */
        paser.line();
        System.out.println(expression + " = " + listener.getResult());
        return listener.getResult();
    }
}